package Lingua::EN::Numbers;

use Exporter;

@ISA = qw( Exporter );
@EXPORT_OK = qw { to_words };
$VERSION = '0.02';

use strict;

$Lingua::EN::Numbers::comma  = ',';
$Lingua::EN::Numbers::and  = 'and';
$Lingua::EN::Numbers::zero = 'zero';
$Lingua::EN::Numbers::decimal = 'point';

my @unit = ('',qw(one two three four five six seven eight nine));
my @teen = qw(ten eleven twelve thirteen fourteen
	      fifteen sixteen seventeen eighteen nineteen);
my @ten  = ('','',qw(twenty thirty forty fifty sixty seventy eighty ninety));
my @mill = map { (my $val=$_) =~ s/_/illion/; " $val" }
	   ('',qw(thousand m_ b_ tr_ quadr_ quint_ sext_ sept_ oct_ non_ dec_));

sub mill { my $ind = $_[0]||0;
	   die "Number out of range\n" if $ind > $#mill;
	   return $ind<@mill ? $mill[$ind] : ' ???illion'; }

sub unit { return $unit[$_[0]]. mill($_[1]); }

sub ten
{
	return $ten[$_[0]] . ($_[0]&&$_[1]?'-':'') . $unit[$_[1]] . mill($_[2])
		if $_[0] ne '1';
	return $teen[$_[1]]. $mill[$_[2]||0];
}

sub hund
{
	return unit($_[0]) . " hundred" . ($_[1] || $_[2] ? " $Lingua::EN::Numbers::and " : '')
	     . ten($_[1],$_[2]) . mill($_[3]) . ', ' if $_[0];
	return ten($_[1],$_[2]) . mill($_[3]) . ', ' if $_[1] || $_[2];
	return '';
}

sub enword
{
	my ($num,$group,$zero,$comma) = @_;

	if ($group==1)
	{
		$num =~ s/(\d)/ ($1 ? unit($1) :" $zero")."$comma " /eg;
	}
	elsif ($group==2)
	{
		$num =~ s/(\d)(\d)/ ($1 ? ten($1,$2) : $2 ? " $zero " . unit($2) : " $zero $zero") . "$comma " /eg;
		$num =~ s/(\d)/ ($1 ? unit($1) :" $zero")."$comma " /e;
	}
	elsif ($group==3)
	{
		$num =~ s/(\d)(\d)(\d)/ ($1 ? unit($1) :" $zero")." ".($2 ? ten($2,$3) : $3 ? " $zero " . unit($3) : " $zero $zero") . "$comma " /eg;
		$num =~ s/(\d)(\d)/ ($1 ? ten($1,$2) : $2 ? " $zero " . unit($2) : " $zero $zero") . "$comma " /e;
		$num =~ s/(\d)/ ($1 ? unit($1) :" $zero")."$comma " /e;
	}
	elsif ($num+0==0)
	{
		$num = $zero;
	}
	else
	{
		$num =~ s/\A\s*0+//;
		my $mill = 0;
		1 while $num =~ s/(\d)(\d)(\d)(?=\D*\Z)/ hund($1,$2,$3,$mill++) /e;
		$num =~ s/(\d)(\d)(?=\D*\Z)/ ten($1,$2,$mill)."$comma " /e;
		$num =~ s/(\d)(?=\D*\Z)/ unit($1,$mill) . "$comma "/e;
	}
	return $num;
}

sub to_words($;$)
{
	my ($num,$group) = (@_,0);
	die "Bad chunking option: $group\n" unless $group =~ /\A[0-3]\Z/;
	my $sign = ($num =~ /\A\s*\+/) ? "plus"
		 : ($num =~ /\A\s*\-/) ? "minus"
		 : '';

	my $zero = $Lingua::EN::Numbers::zero;
	my $comma = $Lingua::EN::Numbers::comma;

	my @chunks = ($Lingua::EN::Numbers::decimal)
			? $group ? split(/\./, $num) : split(/\./, $num, 2)
			: ($num);

	my $first = 1;

	if ($chunks[0] eq '') { $first=0; shift @chunks; }

	foreach ( @chunks )
	{
		s/\D//g;
		$_ = '0' unless $_;

		if (!$group && !$first) { $_ = enword($_,1,$zero,$comma) }
		else         	        { $_ = enword($_,$group,$zero,$comma) }

		s/, \Z//;
		s/\s+,/,/g;
		s/, (\S+)\s+\Z/ $Lingua::EN::Numbers::and $1/ if !$group and $first;
		s/\s+/ /g;
		s/(\A\s|\s\Z)//g;
		$first = '' if $first;
	}

	my @numchunks = ();
	if ($first =~ /0/)
	{
		unshift @chunks, '';
	}
	else
	{
		@numchunks = split /\Q$comma /, $chunks[0];
	}

	foreach (@chunks[1..$#chunks])
	{
		push @numchunks, $Lingua::EN::Numbers::decimal;
		push @numchunks, split /\Q$comma /;
	}

	if (wantarray)
	{
		unshift @numchunks, $sign if $sign;
		return @numchunks
	}
	elsif ($group)
	{
		return ($sign?"$sign ":'') .  join ", ", @numchunks;
	}
	else
	{
		$num = ($sign?"$sign ":'') . shift @numchunks;
		$first = ($num !~ /$Lingua::EN::Numbers::decimal\Z/);
		foreach ( @numchunks )
		{
			if (/\A$Lingua::EN::Numbers::decimal\Z/)
			{
				$num .= " $_";
				$first = 0;
			}
			elsif ($first)
			{
				$num .= "$comma $_";
			}
			else
			{
				$num .= " $_";
			}
		}
		return $num;
	}
}

1;
__END__

=head1 NAME

Lingua::EN::Numbers - Convert numbers to English words


=head1 SYNOPSIS

    use Lingua::EN::Numbers qw ( to_words );

 # SCALAR CONTEXTS GET BACK A SINGLE STRING...

    $words = to_words(1234);	# "one thousand, two hundred and thirty-four"


 # LIST CONTEXTS GET BACK A LIST OF STRINGS CORRESPONDING TO EACH "CHUNK"...

    @words = to_words(1234);	# ("one thousand","two hundred and thirty-four")


 # THE OPTIONAL SECOND PARAMETER (AN INTEGER IN THE RANGE [1-3])
 # CHANGES THE WAY A NUMBER IS "CHUNKED"...

    $words = to_word(12345,1);	# "one, two, three, four, five"
    $words = to_word(12345,2);	# "twelve, thirty-four, five"
    $words = to_word(12345,3);	# "one twenty-three, forty-five"


 # THE PACKAGE VARIABLE $Lingua::EN::Numbers::and
 # CONTROLS HOW "and" IS INSERTED...

    $Lingua::EN::Numbers::and = "";
    $words = to_words(1234);	# "one thousand, two hundred thirty-four"


 # THE PACKAGE VARIABLE $Lingua::EN::Numbers::zero
 # CONTROLS HOW ZERO IS TRANSLATED...

    $Lingua::EN::Numbers::zero = "oh";
    $words = to_words(555 1202,1);	# "five, five, five, one, two, oh, two"


=head1 DESCRIPTION

The exportable subroutine C<Lingua::EN::Numbers::to_words($;$)> takes a number
and returns an English representation of that number. In a scalar context 
a string is returned. Hence:

	use Lingua::EN::Numbers qw( to_words );

	$words = to_words(1234567);

puts the string:

	"one million, two hundred and thirty-four thousand, five hundred and sixty-seven"
	
into $words.

In a list context each comma-separated chunk is returned as a separate element.
Hence:

	@words = to_words(1234567);

puts the list:

	("one million",
	 "two hundred and thirty-four thousand",
	 "five hundred and sixty-seven")

into @words.

Non-digits (apart from an optional leading plus or minus sign and any
decimal points) are silently ignored, so the following all produce
identical results:

	to_words(5551202);
	to_words(5_551_202);
	to_words("5,551,202");
	to_words("555-1202");

That last case is a little awkward since it's almost certainly a phone number,
and "five million, five hundred and fifty-one thousand, two hundred and two"
probably isn't what's wanted.

To overcome this, <to_words()> takes an optional second argument which
changes how numbers are translated. The argument must be a positive
integer less than four, which indicated how the digits of the number
are to be "chunked". If the argument is C<1>, then each digit is
translated separately. If the argument is C<2>, pairs of digits
(starting from the I<left>) are chunked together. If the argument is
C<3>, triples of numbers (again, from the I<left>) are chunked. Hence:

	to_words("555-1202",1)

returns C<"five, five, five, one, two, zero, two">, whilst:

	to_words("555-1202",2)

returns C<"fifty-five, fifty-one, twenty, two">, and:

	to_words("555-1202",3)

returns C<"five fifty-five, one twenty, two">.

Phone numbers are often written in words as
C<"five..five..five..one..two..zero..two">, which is also easy to
achieve:

	join '..', to_words("555-1202")

C<Lingua::EN::Numbers> also handles decimal fractions. Hence:

	to_words("1.2345")

returns C<"one point two three four five"> in a scalar context
and C<("one","point","two","three","four","five")>) in an array context.
Exponent form (C<"1.234e56">) is not yet handled.

Multiple decimal points are only translated in one of the "chunking" modes.
Hence:

	to_words(101.202.303)

returns C<"one hundred and one point two zero two three zero three">,
whereas:

	to_words(101.202.303,3)

returns C<"one zero one, point, two zero two, point, three zero three">.

The digit C<'0'> is unusual in that in may be translated to English as "zero",
"oh", or "nought". To cater for this diversity, C<Lingua::EN::Numbers>
provides a package variable (C<$Lingua::EN::Numbers::zero>) which may be set to
the desired translation of C<'0'>. For example:

	$Lingua::EN::Numbers::zero = "oh";
	print join "..", to_words("555-1202",3)

print C<"five..five..five..one..two..oh..two">.
By default, zero is rendered as "zero".

Another major regional variation in number translation is the use of
"and" in certain contexts. The package variable
C<$Lingua::EN::Numbers::and> allows the programmer to specify how "and" should
be handled. Hence:

	$Lingua::EN::Numbers::and = '';
	print scalar to_words("765")

prints "seven hundred sixty-five", instead of "seven hundred and sixty-five".
By default, the "and" is included.

The translation of the decimal point is also subject to variation
(with "point", "dot", and "decimal" being the favorites).
The package variable C<$Lingua::EN::Numbers::decimal> allows the
programmer to how the decimal point should be rendered. Hence:

	$Lingua::EN::Numbers::decimal = 'dot';
		print scalar to_words("666.124.64.101",3)

prints "six sixty-six, dot, one twenty-four, dot, sixty-four, dot, one zero one"
By default, the decimal point is rendered as "point".


=head1 DIAGNOSTICS

C<Lingua::EN::Numbers::to_words()> can C<die> with the following messages:

=over 8

=item C<"Bad chunking option: %s">

The optional argument to C<to_words()> wasn't 1, 2 or 3.

=item C<"Number out of range">

C<to_words()> was passed a number larger than
999,999,999,999,999,999,999,999,999,999,999,999 (that is: nine hundred
and ninety-nine decillion, nine hundred and ninety-nine nonillion, nine
hundred and ninety-nine octillion, nine hundred and ninety-nine
septillion, nine hundred and ninety-nine sextillion, nine hundred and
ninety-nine quintillion, nine hundred and ninety-nine quadrillion, nine
hundred and ninety-nine trillion, nine hundred and ninety-nine billion,
nine hundred and ninety-nine million, nine hundred and ninety-nine
thousand, nine hundred and ninety-nine :-) 

The problem is that C<Lingua::EN::Numbers::to_words> doesn't know any
words for number components bigger than "decillion".

=back

=head1 AUTHOR

Damian Conway (damian@cs.monash.edu.au)


=head1 BUGS AND IRRITATIONS

There are undoubtedly serious bugs lurking somewhere in this code.
Bug reports and other feedback are most welcome.


=head1 COPYRIGHT

 Copyright (c) 1998, Damian Conway. All Rights Reserved.
 This module is free software. It may be used, redistributed
and/or modified under the terms of the Perl Artistic License
     (see http://www.perl.com/perl/misc/Artistic.html)


